/* Copyright (c) 2020, The Linux Foundation. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above
 *       copyright notice, this list of conditions and the following
 *       disclaimer in the documentation and/or other materials provided
 *       with the distribution.
 *     * Neither the name of The Linux Foundation nor the names of its
 *       contributors may be used to endorse or promote products derived
 *       from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 * IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef NTRIP_CLIENT_INTEGRATION_API_H
#define NTRIP_CLIENT_INTEGRATION_API_H

#include <functional>

#ifdef __cplusplus
extern "C" {
#endif

/** @fn
    @brief
    CorrectionDataCb is for receiving correction data from NTRIP client.
*/
typedef std::function<void(uint8_t *correctionData, uint32_t lengthInBytes)>
        CorrectionDataCb;

/** @brief API to start streaming for correction data.
PPE service shall call this API when it is ready to start receiving correction data.
*/
void startCorrectionDataStreaming(CorrectionDataCb corrDataCb);

/** @brief API to stop streaming for correction data.
PPE service shall call this API when it wants to stop receiving correction data.
*/
void stopCorrectionDataStreaming();

#ifdef __cplusplus
} // extern "C"
#endif

#endif //NTRIP_CLIENT_INTEGRATION_API_H
